@@ -82,15 +82,15 @@ func (p *interp) toNative(v value, typ reflect.Type) reflect.Value {
 	case reflect.Int64:
 		return reflect.ValueOf(int64(v.num()))
 	case reflect.Uint:
-		return reflect.ValueOf(uint(v.num()))
+		return reflect.ValueOf(uint(int64(v.num())))
 	case reflect.Uint8:
-		return reflect.ValueOf(uint8(v.num()))
+		return reflect.ValueOf(uint8(int64(v.num())))
 	case reflect.Uint16:
-		return reflect.ValueOf(uint16(v.num()))
+		return reflect.ValueOf(uint16(int64(v.num())))
 	case reflect.Uint32:
-		return reflect.ValueOf(uint32(v.num()))
+		return reflect.ValueOf(uint32(int64(v.num())))
 	case reflect.Uint64:
-		return reflect.ValueOf(uint64(v.num()))
+		return reflect.ValueOf(uint64(int64(v.num())))
 	case reflect.Float32:
 		return reflect.ValueOf(float32(v.num()))
 	case reflect.Float64:
@@ -363,10 +363,21 @@ func (p *interp) parseFmtTypes(s string) (format string, types []byte, err error
 			switch s[i] {
 			case 's':
 				t = 's'
-			case 'd', 'i', 'o', 'x', 'X':
+			case 'd':
 				t = 'd'
+			case 'o', 'x', 'X':
+				t = 'u'
+			case 'i':
+				t = 'd'
+				out[i] = 'd'
 			case 'f', 'e', 'E', 'g', 'G':
 				t = 'f'
+			case 'a':
+				t = 'f'
+				out[i] = 'x'
+			case 'A':
+				t = 'f'
+				out[i] = 'X'
 			case 'u':
 				t = 'u'
 				out[i] = 'd'
@@ -405,25 +416,26 @@ func (p *interp) sprintf(format string, args []value) (string, error) {
 		case 's':
 			v = p.toString(a)
 		case 'd':
-			v = int(a.num())
+			v = int64(a.num())
 		case 'f':
 			v = a.num()
 		case 'u':
-			v = uint(a.num())
+			v = uint64(int64(a.num()))
 		case 'c':
 			var c []byte
 			n, isStr := a.isTrueStr()
 			if isStr {
 				s := p.toString(a)
-				if len(s) > 0 {
-					c = []byte{s[0]}
+				_, size := utf8.DecodeRuneInString(s)
+				if size > 0 {
+					c = []byte(s[:size])
 				} else {
 					c = []byte{0}
 				}
 			} else {
-				// Follow the behaviour of awk and mawk, where %c
-				// operates on bytes (0-255), not Unicode codepoints
-				c = []byte{byte(n)}
+				c = make([]byte, utf8.UTFMax)
+				size := utf8.EncodeRune(c, rune(n))
+				c = c[:size]
 			}
 			v = c
 		}
