@@ -980,7 +980,7 @@ func (p *interp) callBuiltin(builtinOp compiler.BuiltinOp) error {
 		p.replaceTop(num(float64(index + 1)))
 
 	case compiler.BuiltinInt:
-		p.replaceTop(num(float64(int(p.peekTop().num()))))
+		p.replaceTop(num(float64(int64(p.peekTop().num()))))
 
 	case compiler.BuiltinLength:
 		p.push(num(float64(len(p.line))))
@@ -1021,7 +1021,8 @@ func (p *interp) callBuiltin(builtinOp compiler.BuiltinOp) error {
 
 	case compiler.BuiltinSrand:
 		prevSeed := p.randSeed
-		p.random.Seed(time.Now().UnixNano())
+		p.randSeed = float64(time.Now().Unix())
+		p.random.Seed(int64(math.Float64bits(p.randSeed)))
 		p.push(num(prevSeed))
 
 	case compiler.BuiltinSrandSeed:
